#!/usr/bin/env node
"use strict";

var _readline = _interopRequireDefault(require("readline"));

var _env = require("./env.js");

var _common = require("./common.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const watcherReady = () => {
  console.log('Watching ' + _env.WATCH_PATTERNS.map(p => p.replace(_env.APP_DIR + '/', '')).join(', ') + ` in ${_env.APP_DIR} for changes.`);
};

const reloadStart = relPath => {
  console.log(`\n${relPath} changed`);
  console.log('Starting code reload..');
};

const reloadEnd = () => {
  console.log('Reload done');
};

(0, _common.checkIsAppDir)();
const containers = (0, _common.dockerContainers)();

if (Object.keys(containers).length == 0) {
  console.log("No running containers. Exiting ...");
  process.exit();
}

const watcher = (0, _common.runWatcher)(containers, console, watcherReady, reloadStart, reloadEnd);
console.log("You can reset the db by pressing the 'r' button\n");

_readline.default.emitKeypressEvents(process.stdin);

process.stdin.setRawMode(true);
process.stdin.on('keypress', (str, key) => {
  if (key.name === "c" && key.ctrl) process.exit();

  if (key.name === "r") {
    (0, _common.resetDb)(containers, console);
  }
});
process.on('exit', () => watcher.close());